@@ -438,6 +438,7 @@ class TokenController(wsgi.Application):
             self.token_api.create_token(
                 context, token_id, dict(key=token_id,
                                         id=token_id,
+                                        expires=auth_token_data['expires'],
                                         user=user_ref,
                                         tenant=tenant_ref,
                                         metadata=metadata_ref))
@@ -623,7 +624,7 @@ class TokenController(wsgi.Application):
 
         """
         if not catalog_ref:
-            return {}
+            return []
 
         services = {}
         for region, region_ref in catalog_ref.iteritems():
