@@ -19,10 +19,11 @@ import copy
 
 import memcache
 
+from keystone.common import utils
 from keystone import config
 from keystone import exception
+from keystone.openstack.common import jsonutils
 from keystone import token
-from keystone.common import utils
 
 
 CONF = config.CONF
@@ -30,6 +31,8 @@ config.register_str('servers', group='memcache', default='localhost:11211')
 
 
 class Token(token.Driver):
+    revocation_key = 'revocation-list'
+
     def __init__(self, client=None):
         self._memcache_client = client
 
@@ -45,6 +48,9 @@ class Token(token.Driver):
     def _prefix_token_id(self, token_id):
         return 'token-%s' % token_id.encode('utf-8')
 
+    def _prefix_user_id(self, user_id):
+        return 'usertokens-%s' % user_id.encode('utf-8')
+
     def get_token(self, token_id):
         ptk = self._prefix_token_id(token_id)
         token = self.client.get(ptk)
@@ -64,9 +70,9 @@ class Token(token.Driver):
             kwargs['time'] = expires_ts
         self.client.set(ptk, data_copy, **kwargs)
         if 'id' in data['user']:
-            token_data = token_id
+            token_data = jsonutils.dumps(token_id)
             user_id = data['user']['id']
-            user_key = 'usertokens-%s' % user_id
+            user_key = self._prefix_user_id(user_id)
             if not self.client.append(user_key, ',%s' % token_data):
                 if not self.client.add(user_key, token_data):
                     if not self.client.append(user_key, ',%s' % token_data):
@@ -74,20 +80,43 @@ class Token(token.Driver):
                         raise exception.UnexpectedError(msg)
         return copy.deepcopy(data_copy)
 
+    def _add_to_revocation_list(self, data):
+        data_json = jsonutils.dumps(data)
+        if not self.client.append(self.revocation_key, ',%s' % data_json):
+            if not self.client.add(self.revocation_key, data_json):
+                if not self.client.append(self.revocation_key,
+                                          ',%s' % data_json):
+                    msg = _('Unable to add token to revocation list.')
+                    raise exception.UnexpectedError(msg)
+
     def delete_token(self, token_id):
         # Test for existence
-        self.get_token(token_id)
+        data = self.get_token(token_id)
         ptk = self._prefix_token_id(token_id)
         result = self.client.delete(ptk)
+        self._add_to_revocation_list(data)
         return result
 
-    def list_tokens(self, user_id):
+    def list_tokens(self, user_id, tenant_id=None):
         tokens = []
-        user_record = self.client.get('usertokens-%s' % user_id) or ""
-        token_list = user_record.split(',')
+        user_key = self._prefix_user_id(user_id)
+        user_record = self.client.get(user_key) or ""
+        token_list = jsonutils.loads('[%s]' % user_record)
         for token_id in token_list:
             ptk = self._prefix_token_id(token_id)
-            token = self.client.get(ptk)
-            if token:
+            token_ref = self.client.get(ptk)
+            if token_ref:
+                if tenant_id is not None:
+                    tenant = token_ref.get('tenant')
+                    if not tenant:
+                        continue
+                    if tenant.get('id') != tenant_id:
+                        continue
                 tokens.append(token_id)
         return tokens
+
+    def list_revoked_tokens(self):
+        list_json = self.client.get(self.revocation_key)
+        if list_json:
+            return jsonutils.loads('[%s]' % list_json)
+        return []
